/*
** Module   : NativeBuffer.java
** Abstract : descriptor for memptr as an argument or return value to a native function 
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20140107 First version.
*/

package com.goldencode.p2j.library;

import java.io.*;
import java.math.*;

import com.goldencode.p2j.util.*;

/**
 * A descriptor for memptr as an argument or return value to a native function.
 */
public class NativeBuffer
extends NativePrimitive
{
   /** The stored value. */
   protected memptr value; 
   
   /**
    * Default constructor.
    */
   public NativeBuffer()
   {
      super();
      type = LibraryManager.TYPE_POINTER;
   }
   
   /**
    * Access the data element.
    *
    * @return   The buffer.
    */
   public memptr getValue()
   {
      return value;
   }
   
   /**
    * Assign the data element.
    *
    * @param    value
    *           The buffer.
    */
   public void setValue(memptr value)
   {
      this.value = value;
   }
   
   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      value = (memptr) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeObject(value);
   }
   
   /**
    * Report the size (in bytes) of the native memory needed to store this instance.
    * <p>
    * This should never be called on the server, only on the client side.
    *
    * @return   The number of bytes needed.
    */
   public int size()
   {
      return (value == null || value.isUnknown()) ? 0 : (int) value.lengthOf();
   }
   
   /**
    * Worker method to read the primitive value from the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void readNative(memptr ptr)
   {
      if (value == null)
         value = new memptr();
      
      value.assign(ptr);
   }
   
   /**
    * Worker method to write the primitive value to the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void writeNative(memptr ptr)
   {
      ptr.assign(value);
   }
}
