/*
** Module   : Parameter.java
** Abstract : internal entry parameter container 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20131124 Moved this code out of SourceNameMapper into its own class. This enables
**                  cleaner separation and modularity. Added synchronization and made changes
**                  for RETURN support.
*/

package com.goldencode.p2j.util;

import java.util.*;

/**
 * Container for all info related to an internal-entry's {@link InternalEntry} parameter.
 */
public class Parameter
{
   /** Constant identifying no-extent parameters. */
   private static final int PARAM_NO_EXTENT = -2;
   
   /** Constant identifying dynamic-extent parameters. */
   private static final int PARAM_DYNAMIC_EXTENT = -1;
   
   /** The set of all valid modes. */
   private static final Set<String> validModes = new HashSet<String>();
   
   /** Legacy 4GL name for this parameter. */
   private final String pname;

   /** Converted Java name for this parameter. */
   private final String jname;

   /** The 4GL-style parameter type. */
   private final String type;
   
   /** The mode of this parameter, one of INPUT, OUTPUT or INPUT-OUTPUT or RETURN. */
   private final String mode;
   
   /** A map containing all the parameter's attributes. */
   private Map<String, String> attributes = new HashMap<String, String>();

   /**
    * Extent setting of this parameter. If set to {@link #PARAM_DYNAMIC_EXTENT}, dynamic extent 
    * is used.  If set to {@link #PARAM_NO_EXTENT}, no extent is used.
    */
   private int extent;

   static
   {
      validModes.add("INPUT");
      validModes.add("INPUT-OUTPUT");
      validModes.add("OUTPUT");
      validModes.add("RETURN");
   }

   /**
    * Basic c'tor.
    * 
    * @param    pname
    *           Legacy 4GL name for this parameter.
    * @param    jname
    *           Converted Java name for this parameter.
    * @param    type
    *           The 4GL-style parameter type.
    * @param    mode
    *           The mode of this parameter, one of INPUT, OUTPUT or 
    *           INPUT-OUTPUT.
    */
   public Parameter(String pname, String jname, String type, String mode)
   {
      this.pname = pname;
      this.jname = jname;
      this.type  = type;
      this.mode  = mode.toUpperCase();
      
      if (!"BUFFER".equalsIgnoreCase(type) && !validModes.contains(this.mode))
      {
         String errmsg = "Malformed parameter mode " + mode + " for parameter " + pname;
         throw new IllegalStateException(errmsg);
      }
   }
   
   /**
    * Get the legacy 4GL name.
    *
    * @return   The legacy name.
    */
   public String getLegacyName()
   {
      return pname;
   }
   
   /**
    * Get the converted name.
    *
    * @return   The converted name.
    */
   public String getConvertedName()
   {
      return jname;
   }
   
   /**
    * Get the unmodified mode name.
    *
    * @return   The raw mode name.
    */
   public String getMode()
   {
      return mode;
   }
   
   /**
    * Get the data type name.
    *
    * @return   The data type name.
    */
   public String getType()
   {
      return type;
   }
   
   /**
    * Get the value of the given attribute.  If the attribute is null or
    * is not set, return <code>null</code>.
    * 
    * @param    attr
    *           The attribute's name.
    *           
    * @return   See above.
    */
   public synchronized String getAttribute(String attr)
   {
      if (attr== null)
      {
         return null;
      }
      
      attr = attr.toLowerCase();
      
      return attributes.get(attr);
   }

   /**
    * Set the value of the given attribute.
    * 
    * @param    name
    *           The attribute's name.
    * @param    value
    *           The attribute's value.
    */
   public synchronized void putAttribute(String name, String value)
   {
      attributes.put(name.toLowerCase(), value);
   }

   /**
    * Get the value of the mode in a form that is acceptable for displaying in the
    * internal entry's signature. In particular, any RETURN mode is reported as OUTPUT.
    * 
    * @return   The mode in a form that is compatible with the 4GL signature text.
    */
   public String getSignatureMode()
   {
      return "RETURN".equals(mode) ? "OUTPUT" : mode;
   }

   /**
    * Check if this parameter is dynamic extent (<code>true</code> only 
    * when the {@link #extent} field is set to {@link #PARAM_DYNAMIC_EXTENT}.
    * 
    * @return   <code>true</code> if this is a dynamic-extent parameter.
    */
   public synchronized boolean isDynamicExtent()
   {
      return extent == PARAM_DYNAMIC_EXTENT;
   }
   
   /**
    * Check if this parameter is fixed-size extent (<code>true</code> only 
    * when the {@link #extent} field greater than 0}.
    * 
    * @return   <code>true</code> if this is a fixed-size extent parameter.
    */
   public synchronized boolean isFixedExtent()
   {
      return extent > 0;
   }
   
   /**
    * Check if this parameter is an extent parameter (fixed or dynamic).
    * 
    * @return   <code>true</code> if this is an extent parameter.
    */
   public synchronized boolean isExtent()
   {
      return isDynamicExtent() || isFixedExtent();
   }
   
   /**
    * Obtain the extent of this parameter.
    *
    * @return   The extent.
    */
   public synchronized int getExtent()
   {
      return isDynamicExtent() ? 0 : extent;
   }
   
   /**
    * Initialize the extent member based on a text input.
    *
    * @param    extent
    *           A valid integer specifying the extent or <code>null</code> to set the default
    *           to {@link #PARAM_NO_EXTENT}.
    */
   public synchronized void setExtent(String extent)
   {
      this.extent = (extent == null) ? PARAM_NO_EXTENT : Integer.parseInt(extent);      
   }
}

